use serde_json::{json, Value};

use svm_sdk::template;

#[template]
mod Template {
    #[storage]
    struct Storage {
        a: bool,
        b: u8,
        c: i8,
        d: u16,
        e: i16,
        f: u32,
        g: i32,
        h: u64,
        i: i64,
        j: Amount,
        k: Address,
        l: [bool; 3],
        m: [u16; 4],
        o: [Amount; 2],
    }
}

fn main() {
    let raw = raw_meta();
    let json: Value = serde_json::from_str(&raw).unwrap();

    assert_eq!(
        json,
        json!({
            "schema": [
                {"id": 0,  "name": "a", "type": "bool",     "offset": 0,  "byte_count": 1},
                {"id": 1,  "name": "b", "type": "u8",       "offset": 1,  "byte_count": 1},
                {"id": 2,  "name": "c", "type": "i8",       "offset": 2,  "byte_count": 1},
                {"id": 3,  "name": "d", "type": "u16",      "offset": 3,  "byte_count": 2},
                {"id": 4,  "name": "e", "type": "i16",      "offset": 5,  "byte_count": 2},
                {"id": 5,  "name": "f", "type": "u32",      "offset": 7,  "byte_count": 4},
                {"id": 6,  "name": "g", "type": "i32",      "offset": 11, "byte_count": 4},
                {"id": 7,  "name": "h", "type": "u64",      "offset": 15, "byte_count": 8},
                {"id": 8,  "name": "i", "type": "i64",      "offset": 23, "byte_count": 8},
                {"id": 9,  "name": "j", "type": "Amount",   "offset": 31, "byte_count": 8},
                {"id": 10, "name": "k", "type": "Address",  "offset": 39, "byte_count": 20},
                {"id": 11, "name": "l", "type": "[bool]",   "offset": 59, "byte_count": 1, "length": 3},
                {"id": 14, "name": "m", "type": "[u16]",    "offset": 62, "byte_count": 2, "length": 4},
                {"id": 18, "name": "o", "type": "[Amount]", "offset": 70, "byte_count": 8, "length": 2},
            ],
            "api": [],
        })
    );
}
